@@ -36,13 +36,23 @@
 /* Do use SetCapture, it works now. Thanks to jpe@archaeopteryx.com */
 #define USE_SETCAPTURE 1
 
-#include <stdio.h>
+#include "config.h"
+
+#include <glib/gprintf.h>
+
+#if defined (__GNUC__) && defined (HAVE_DIMM_H)
+/* The w32api imm.h clashes a bit with the IE5.5 dimm.h */
+# define IMEMENUITEMINFOA hidden_IMEMENUITEMINFOA
+# define IMEMENUITEMINFOW hidden_IMEMENUITEMINFOW
+#endif
 
 #include "gdk.h"
 #include "gdkprivate-win32.h"
 #include "gdkinput-win32.h"
 #include "gdkkeysyms.h"
 
+#include <windowsx.h>
+
 #ifdef G_WITH_CYGWIN
 #include <fcntl.h>
 #include <errno.h>
@@ -50,12 +60,6 @@
 
 #include <objbase.h>
 
-#if defined (__GNUC__) && defined (HAVE_DIMM_H)
-/* The w32api imm.h clashes a bit with the IE5.5 dimm.h */
-# define IMEMENUITEMINFOA hidden_IMEMENUITEMINFOA
-# define IMEMENUITEMINFOW hidden_IMEMENUITEMINFOW
-#endif
-
 #include <imm.h>
 
 #if defined (__GNUC__) && defined (HAVE_DIMM_H)
@@ -169,8 +173,8 @@ real_window_procedure (HWND   hwnd,
   msg.lParam = lparam;
   msg.time = _gdk_win32_get_next_tick (0);
   pos = GetMessagePos ();
-  msg.pt.x = LOWORD (pos);
-  msg.pt.y = HIWORD (pos);
+  msg.pt.x = GET_X_LPARAM (pos);
+  msg.pt.y = GET_Y_LPARAM (pos);
 
   event = gdk_event_new (GDK_NOTHING);
   ((GdkEventPrivate *)event)->flags |= GDK_EVENT_PENDING;
@@ -199,6 +203,7 @@ real_window_procedure (HWND   hwnd,
     return ret_val;
   else
     {
+      GDK_NOTE (EVENTS, g_print("calling DefWindowProc\n"));
 #ifndef HAVE_DIMM_H
       return DefWindowProc (hwnd, message, wparam, lparam);
 #else
@@ -318,7 +323,7 @@ _gdk_events_init (void)
       while (latin_locale == NULL && i < G_N_ELEMENTS (latin_languages))
 	{
 	  char id[9];
-	  sprintf (id, "%08x", MAKELANGID (latin_languages[i++], SUBLANG_DEFAULT));
+	  g_sprintf (id, "%08x", MAKELANGID (latin_languages[i++], SUBLANG_DEFAULT));
 	  latin_locale = LoadKeyboardLayout (id, KLF_NOTELLSHELL|KLF_SUBSTITUTE_OK);
 	}
     }
@@ -451,7 +456,7 @@ event_mask_string (GdkEventMask mask)
   *p = '\0';
 #define BIT(x) \
   if (mask & GDK_##x##_MASK) \
-    p += sprintf (p, "%s" #x, (p > bfr ? " " : ""))
+    p += g_sprintf (p, "%s" #x, (p > bfr ? " " : ""))
   BIT (EXPOSURE);
   BIT (POINTER_MOTION);
   BIT (POINTER_MOTION_HINT);
@@ -527,8 +532,8 @@ gdk_pointer_grab (GdkWindow    *window,
   
   if (!cursor)
     hcursor = NULL;
-  else
-    hcursor = cursor_private->hcursor;
+  else if ((hcursor = CopyCursor (cursor_private->hcursor)) == NULL)
+    WIN32_API_FAILED ("CopyCursor");
 #if 0
   return_val = _gdk_input_grab_pointer (window,
 					owner_events,
@@ -562,6 +567,9 @@ gdk_pointer_grab (GdkWindow    *window,
     {
       p_grab_window = window;
       p_grab_cursor = hcursor;
+
+      if (p_grab_cursor != NULL)
+	SetCursor (p_grab_cursor);
     }
   
   return return_val;
@@ -584,6 +592,13 @@ gdk_display_pointer_ungrab (GdkDisplay *display,
 #endif
 
   p_grab_window = NULL;
+  if (p_grab_cursor != NULL)
+    {
+      if (GetCursor () == p_grab_cursor)
+	SetCursor (NULL);
+      DestroyCursor (p_grab_cursor);
+      p_grab_cursor = NULL;
+    }
 }
 
 /*
@@ -775,6 +790,16 @@ gdk_event_apply_filters (MSG      *msg,
   return GDK_FILTER_CONTINUE;
 }
 
+void 
+gdk_display_add_client_message_filter (GdkDisplay   *display,
+				       GdkAtom       message_type,
+				       GdkFilterFunc func,
+				       gpointer      data)
+{
+  /* XXX */
+  gdk_add_client_message_filter (message_type, func, data);
+}
+
 void 
 gdk_add_client_message_filter (GdkAtom       message_type,
 			       GdkFilterFunc func,
@@ -869,6 +894,8 @@ build_keypress_event (GdkEvent *event,
   event->key.group = 0;		/* ??? */
   event->key.keyval = GDK_VoidSymbol;
   
+  build_key_event_state (event);
+
   if (msg->message == WM_IME_COMPOSITION)
     {
       himc = ImmGetContext (msg->hwnd);
@@ -892,7 +919,11 @@ build_keypress_event (GdkEvent *event,
 	      /* For ASCII control chars, the keyval should be the
 	       * corresponding ASCII character.
 	       */
-	      event->key.keyval = msg->wParam + '@';
+	      if ((event->key.state & GDK_SHIFT_MASK) == 0)
+		event->key.keyval = msg->wParam + '`';
+	      else
+		event->key.keyval = msg->wParam + '@';
+
 	      /* This is needed in case of Alt+nnn or Alt+0nnn (on the numpad)
 	       * where nnn<32
 	       */
@@ -926,8 +957,6 @@ build_keypress_event (GdkEvent *event,
 				    wbuf, G_N_ELEMENTS (wbuf));
     }
 
-  build_key_event_state (event);
-
   /* Build UTF-8 string */
   if (ucount > 0)
     {
@@ -960,12 +989,18 @@ build_keyrelease_event (GdkEvent *event,
   event->key.state = 0;
   event->key.group = 0;		/* ??? */
 
+  build_key_event_state (event);
+
   if (msg->message == WM_CHAR || msg->message == WM_SYSCHAR)
     {
       event->key.hardware_keycode = vk_from_char (msg->wParam);
       if (msg->wParam < ' ')
 	{
-	  event->key.keyval = msg->wParam + '@';
+	  if ((event->key.state & GDK_SHIFT_MASK) == 0)
+	    event->key.keyval = msg->wParam + '`';
+	  else
+	    event->key.keyval = msg->wParam + '@';
+      
 	  event->key.state |= GDK_CONTROL_MASK;
 	}
       else
@@ -982,7 +1017,7 @@ build_keyrelease_event (GdkEvent *event,
       event->key.keyval = GDK_VoidSymbol;
       event->key.hardware_keycode = 0; /* ??? */
     }
-  build_key_event_state (event);
+
   event->key.string = NULL;
   event->key.length = 0;
 }
@@ -1009,17 +1044,6 @@ print_event_state (gint state)
 #undef CASE
 }
 
-static void
-print_window_state (GdkWindowState state)
-{
-#define CASE(bit) if (state & GDK_WINDOW_STATE_ ## bit ) g_print (#bit " ");
-  CASE (WITHDRAWN);
-  CASE (ICONIFIED);
-  CASE (MAXIMIZED);
-  CASE (STICKY);
-#undef CASE
-}
-
 static void
 print_event (GdkEvent *event)
 {
@@ -1127,8 +1151,9 @@ print_event (GdkEvent *event)
       print_event_state (event->scroll.state);
       break;
     case GDK_WINDOW_STATE:
-      print_window_state (event->window_state.changed_mask);
-      print_window_state (event->window_state.new_window_state);
+      g_print ("%s: %s",
+	       _gdk_win32_window_state_to_string (event->window_state.changed_mask),
+	       _gdk_win32_window_state_to_string (event->window_state.new_window_state));
     default:
       /* Nothing */
       break;
@@ -1218,11 +1243,11 @@ synthesize_enter_event (GdkWindow    *window,
   if (!(GDK_WINDOW_OBJECT (window)->event_mask & GDK_ENTER_NOTIFY_MASK))
     return;
 
-  /* Enter events are at LOWORD (msg->lParam), HIWORD
+  /* Enter events are at GET_X_LPARAM (msg->lParam), GET_Y_LPARAM
    * (msg->lParam) in msg->hwnd */
 
-  pt.x = LOWORD (msg->lParam);
-  pt.y = HIWORD (msg->lParam);
+  pt.x = GET_X_LPARAM (msg->lParam);
+  pt.y = GET_Y_LPARAM (msg->lParam);
   if (msg->hwnd != GDK_WINDOW_HWND (window))
     {
       ClientToScreen (msg->hwnd, &pt);
@@ -1462,11 +1487,12 @@ translate_mouse_coords (GdkWindow *window1,
 {
   POINT pt;
 
-  pt.x = LOWORD (msg->lParam);
-  pt.y = HIWORD (msg->lParam);
+  pt.x = GET_X_LPARAM (msg->lParam);
+  pt.y = GET_Y_LPARAM (msg->lParam);
   ClientToScreen (GDK_WINDOW_HWND (window1), &pt);
   ScreenToClient (GDK_WINDOW_HWND (window2), &pt);
   msg->lParam = MAKELPARAM (pt.x, pt.y);
+
   GDK_NOTE (EVENTS, g_print ("...new coords are (%ld,%ld)\n", pt.x, pt.y));
 }
 
@@ -1623,14 +1649,14 @@ decode_key_lparam (LPARAM lParam)
   char *p = buf;
 
   if (HIWORD (lParam) & KF_UP)
-    p += sprintf (p, "KF_UP ");
+    p += g_sprintf (p, "KF_UP ");
   if (HIWORD (lParam) & KF_REPEAT)
-    p += sprintf (p, "KF_REPEAT ");
+    p += g_sprintf (p, "KF_REPEAT ");
   if (HIWORD (lParam) & KF_ALTDOWN)
-    p += sprintf (p, "KF_ALTDOWN ");
+    p += g_sprintf (p, "KF_ALTDOWN ");
   if (HIWORD (lParam) & KF_EXTENDED)
-    p += sprintf (p, "KF_EXTENDED ");
-  p += sprintf (p, "sc:%d rep:%d", LOBYTE (HIWORD (lParam)), LOWORD (lParam));
+    p += g_sprintf (p, "KF_EXTENDED ");
+  p += g_sprintf (p, "sc:%d rep:%d", LOBYTE (HIWORD (lParam)), LOWORD (lParam));
 
   return buf;
 }
@@ -1875,6 +1901,23 @@ gdk_event_translate (GdkDisplay *display,
   if (ret_val_flagp)
     *ret_val_flagp = FALSE;
 
+  /* init these, since the done: block uses them */
+  ASSIGN_WINDOW (NULL);
+  event->any.window = NULL;
+
+  if (_gdk_default_filters)
+    {
+      /* Apply global filters */
+      GdkFilterReturn result;
+      result = gdk_event_apply_filters (msg, event, _gdk_default_filters);
+      
+      if (result != GDK_FILTER_CONTINUE)
+        {
+          return_val = (result == GDK_FILTER_TRANSLATE) ? TRUE : FALSE;
+          goto done;
+        }
+    }  
+
   ASSIGN_WINDOW (gdk_win32_handle_table_lookup ((GdkNativeWindow) msg->hwnd));
   orig_window = window;
   
@@ -1969,8 +2012,8 @@ gdk_event_translate (GdkDisplay *display,
        * around that. Also, the position is in screen coordinates, not
        * client coordinates as with the button messages.
        */
-      pt.x = LOWORD (msg->lParam);
-      pt.y = HIWORD (msg->lParam);
+      pt.x = GET_X_LPARAM (msg->lParam);
+      pt.y = GET_Y_LPARAM (msg->lParam);
       if ((hwnd = WindowFromPoint (pt)) == NULL)
 	goto done;
 
@@ -2008,8 +2051,8 @@ gdk_event_translate (GdkDisplay *display,
       _gdk_windowing_window_get_offsets (window, &xoffset, &yoffset);
       event->scroll.x = (gint16) pt.x + xoffset;
       event->scroll.y = (gint16) pt.y + yoffset;
-      event->scroll.x_root = (gint16) LOWORD (msg->lParam);
-      event->scroll.y_root = (gint16) HIWORD (msg->lParam);
+      event->scroll.x_root = (gint16) GET_X_LPARAM (msg->lParam);
+      event->scroll.y_root = (gint16) GET_Y_LPARAM (msg->lParam);
       event->scroll.state = 0;	/* No state information with MSH_MOUSEWHEEL */
       event->scroll.device = display->core_pointer;
       return_val = !GDK_WINDOW_DESTROYED (window);
@@ -2361,7 +2404,20 @@ gdk_event_translate (GdkDisplay *display,
 	  break;
 	default:
 	  if (msg->message == WM_SYSKEYDOWN || msg->message == WM_SYSKEYUP)
-	    event->key.keyval = msg->wParam;
+	    {
+	      if (msg->wParam >= 'A' && msg->wParam <= 'Z')
+		{
+		  /* If Alt-unshifted ASCII letter, lowercase */
+		  if (GetKeyState (VK_SHIFT) < 0)
+		    event->key.keyval = msg->wParam;
+		  else
+		    event->key.keyval = msg->wParam + 0x20;
+		}
+	      else
+		{
+		  event->key.keyval = msg->wParam;
+		}
+	    }
 	  else
 	    ignore_wm_char = FALSE;
 	  break;
@@ -2382,14 +2438,10 @@ gdk_event_translate (GdkDisplay *display,
 			 GDK_KEY_PRESS : GDK_KEY_RELEASE);
       event->key.time = _gdk_win32_get_next_tick (msg->time);
       event->key.state = 0;
-      if (GetKeyState (VK_SHIFT) < 0)
-	event->key.state |= GDK_SHIFT_MASK;
-      if (GetKeyState (VK_CAPITAL) & 0x1)
-	event->key.state |= GDK_LOCK_MASK;
-      if (GetKeyState (VK_CONTROL) < 0)
-	event->key.state |= GDK_CONTROL_MASK;
-      if (msg->wParam != VK_MENU && GetKeyState (VK_MENU) < 0)
-	event->key.state |= GDK_MOD1_MASK;
+      build_key_event_state (event);
+      /* Reset MOD1_MASK if it is the Alt key itself */
+      if (msg->wParam == VK_MENU)
+	event->key.state &= ~GDK_MOD1_MASK;
       event->key.hardware_keycode = msg->wParam;
       event->key.group = 0;
       event->key.string = NULL;
@@ -2425,6 +2477,13 @@ gdk_event_translate (GdkDisplay *display,
       if (ignore_wm_char)
 	{
 	  ignore_wm_char = FALSE;
+	  
+	  if (msg->wParam != VK_SPACE && ret_val_flagp)
+	    {
+	      *ret_val_flagp = TRUE;
+	      *ret_valp = FALSE;
+	    }
+	  
 	  break;
 	}
 
@@ -2488,7 +2547,7 @@ gdk_event_translate (GdkDisplay *display,
 		g_print ("WM_%cBUTTONDOWN: %p  (%d,%d)\n",
 			 " LMR"[button],
 			 msg->hwnd,
-			 LOWORD (msg->lParam), HIWORD (msg->lParam)));
+			 GET_X_LPARAM (msg->lParam), GET_Y_LPARAM (msg->lParam)));
 
       if (GDK_WINDOW_OBJECT (window)->extension_events != 0
 	  && _gdk_input_ignore_core)
@@ -2528,10 +2587,10 @@ gdk_event_translate (GdkDisplay *display,
       event->button.time = _gdk_win32_get_next_tick (msg->time);
       if (window != orig_window)
 	translate_mouse_coords (orig_window, window, msg);
-      event->button.x = current_x = (gint16) LOWORD (msg->lParam);
-      event->button.y = current_y = (gint16) HIWORD (msg->lParam);
+      event->button.x = current_x = (gint16) GET_X_LPARAM (msg->lParam);
+      event->button.y = current_y = (gint16) GET_Y_LPARAM (msg->lParam);
       _gdk_windowing_window_get_offsets (window, &xoffset, &yoffset);
-      event->button.x += xoffset;  /* XXX translate current_x, y too? */
+      event->button.x += xoffset;
       event->button.y += yoffset;
       event->button.x_root = current_x_root = msg->pt.x;
       event->button.y_root = current_y_root = msg->pt.y;
@@ -2557,7 +2616,7 @@ gdk_event_translate (GdkDisplay *display,
 		g_print ("WM_%cBUTTONUP: %p  (%d,%d)\n",
 			 " LMR"[button],
 			 msg->hwnd,
-			 LOWORD (msg->lParam), HIWORD (msg->lParam)));
+			 GET_X_LPARAM (msg->lParam), GET_Y_LPARAM (msg->lParam)));
 
       ASSIGN_WINDOW (find_window_for_pointer_event (window, msg));
 
@@ -2585,9 +2644,11 @@ gdk_event_translate (GdkDisplay *display,
 	  event->button.time = _gdk_win32_get_next_tick (msg->time);
 	  if (window != orig_window)
 	    translate_mouse_coords (orig_window, window, msg);
+	  event->button.x = current_x = (gint16) GET_X_LPARAM (msg->lParam);
+	  event->button.y = current_y = (gint16) GET_Y_LPARAM (msg->lParam);
 	  _gdk_windowing_window_get_offsets (window, &xoffset, &yoffset);
-	  event->button.x = (gint16) LOWORD (msg->lParam) + xoffset;
-	  event->button.y = (gint16) HIWORD (msg->lParam) + yoffset;
+	  event->button.x += xoffset;
+	  event->button.y += yoffset;
 	  event->button.x_root = current_x_root = msg->pt.x;
 	  event->button.y_root = current_y_root = msg->pt.y;
 	  event->button.axes = NULL;
@@ -2608,19 +2669,10 @@ gdk_event_translate (GdkDisplay *display,
       GDK_NOTE (EVENTS,
 		g_print ("WM_MOUSEMOVE: %p  %#x (%d,%d)\n",
 			 msg->hwnd, msg->wParam,
-			 LOWORD (msg->lParam), HIWORD (msg->lParam)));
+			 GET_X_LPARAM (msg->lParam), GET_Y_LPARAM (msg->lParam)));
 
       ASSIGN_WINDOW (find_window_for_pointer_event (window, msg));
 
-      /* If we haven't moved, don't create any event.
-       * Windows sends WM_MOUSEMOVE messages after button presses
-       * even if the mouse doesn't move. This disturbs gtk.
-       */
-      if (window == current_window
-	  && LOWORD (msg->lParam) == current_x
-	  && HIWORD (msg->lParam) == current_y)
-	break;
-
       /* HB: only process mouse move messages if we own the active window. */
       GetWindowThreadProcessId (GetActiveWindow (), &pidActWin);
       GetWindowThreadProcessId (msg->hwnd, &pidThis);
@@ -2648,8 +2700,18 @@ gdk_event_translate (GdkDisplay *display,
       event->motion.time = _gdk_win32_get_next_tick (msg->time);
       if (window != orig_window)
 	translate_mouse_coords (orig_window, window, msg);
-      event->motion.x = current_x = (gint16) LOWORD (msg->lParam);
-      event->motion.y = current_y = (gint16) HIWORD (msg->lParam);
+
+      /* If we haven't moved, don't create any event.
+       * Windows sends WM_MOUSEMOVE messages after button presses
+       * even if the mouse doesn't move. This disturbs gtk.
+       */
+      if (window == current_window
+	  && GET_X_LPARAM (msg->lParam) == current_x
+	  && GET_Y_LPARAM (msg->lParam) == current_y)
+	break;
+
+      event->motion.x = current_x = (gint16) GET_X_LPARAM (msg->lParam);
+      event->motion.y = current_y = (gint16) GET_Y_LPARAM (msg->lParam);
       _gdk_windowing_window_get_offsets (window, &xoffset, &yoffset);
       event->motion.x += xoffset;
       event->motion.y += yoffset;
@@ -2665,9 +2727,9 @@ gdk_event_translate (GdkDisplay *display,
 
     case WM_NCMOUSEMOVE:
       GDK_NOTE (EVENTS,
-		g_print ("WM_NCMOUSEMOVE: %p  x,y: %d %d\n",
+		g_print ("WM_NCMOUSEMOVE: %p (%d,%d)\n",
 			 msg->hwnd,
-			 LOWORD (msg->lParam), HIWORD (msg->lParam)));
+			 GET_X_LPARAM (msg->lParam), GET_Y_LPARAM (msg->lParam)));
       if (current_window != NULL
 	  && (GDK_WINDOW_OBJECT (current_window)->event_mask & GDK_LEAVE_NOTIFY_MASK))
 	{
@@ -2709,8 +2771,8 @@ gdk_event_translate (GdkDisplay *display,
        * coordinates as with the button messages. I love the
        * consistency of Windows.
        */
-      pt.x = LOWORD (msg->lParam);
-      pt.y = HIWORD (msg->lParam);
+      pt.x = GET_X_LPARAM (msg->lParam);
+      pt.y = GET_Y_LPARAM (msg->lParam);
       if ((hwnd = WindowFromPoint (pt)) == NULL)
 	break;
 
@@ -2748,8 +2810,8 @@ gdk_event_translate (GdkDisplay *display,
       _gdk_windowing_window_get_offsets (window, &xoffset, &yoffset);
       event->scroll.x = (gint16) pt.x + xoffset;
       event->scroll.y = (gint16) pt.y + yoffset;
-      event->scroll.x_root = (gint16) LOWORD (msg->lParam);
-      event->scroll.y_root = (gint16) HIWORD (msg->lParam);
+      event->scroll.x_root = (gint16) GET_X_LPARAM (msg->lParam);
+      event->scroll.y_root = (gint16) GET_Y_LPARAM (msg->lParam);
       event->scroll.state = build_pointer_event_state (msg);
       event->scroll.device = display->core_pointer;
       return_val = !GDK_WINDOW_DESTROYED (window);
@@ -2946,6 +3008,10 @@ gdk_event_translate (GdkDisplay *display,
       if (!(private->event_mask & GDK_STRUCTURE_MASK))
 	break;
 
+      if (msg->lParam == SW_OTHERUNZOOM ||
+	  msg->lParam == SW_OTHERZOOM)
+	break;
+
       event->any.type = (msg->wParam ? GDK_MAP : GDK_UNMAP);
       event->any.window = window;
 
@@ -2971,13 +3037,9 @@ gdk_event_translate (GdkDisplay *display,
 			     (msg->wParam == SIZE_RESTORED ? "RESTORED" : "?"))))),
 			 LOWORD (msg->lParam), HIWORD (msg->lParam)));
 
-      if (!(private->event_mask & GDK_STRUCTURE_MASK))
-	break;
-
       if (msg->wParam == SIZE_MINIMIZED)
 	{
-	  event->any.type = GDK_UNMAP;
-	  event->any.window = window;
+	  /* Don't generate any GDK event. This is *not* an UNMAP. */
 
 	  if (p_grab_window == window)
 	    gdk_pointer_ungrab (msg->time);
@@ -2985,12 +3047,9 @@ gdk_event_translate (GdkDisplay *display,
 	  if (k_grab_window == window)
 	    gdk_keyboard_ungrab (msg->time);
 
-	  if (window && GDK_WINDOW_IS_MAPPED (window))
-	    gdk_synthesize_window_state (window,
-					 GDK_WINDOW_STATE_MAXIMIZED,
-					 GDK_WINDOW_STATE_ICONIFIED);
-
-	  return_val = !GDK_WINDOW_DESTROYED (window);
+	  gdk_synthesize_window_state (window,
+				       GDK_WINDOW_STATE_WITHDRAWN,
+				       GDK_WINDOW_STATE_ICONIFIED);
 	}
       else if ((msg->wParam == SIZE_RESTORED
 		|| msg->wParam == SIZE_MAXIMIZED)
@@ -2999,6 +3058,12 @@ gdk_event_translate (GdkDisplay *display,
 #endif
 								 )
 	{
+	  GdkWindowState withdrawn_bit =
+	    IsWindowVisible (msg->hwnd) ? GDK_WINDOW_STATE_WITHDRAWN : 0;
+
+	  if (!(private->event_mask & GDK_STRUCTURE_MASK))
+	    break;
+
 	  event->configure.type = GDK_CONFIGURE;
 	  event->configure.window = window;
 	  pt.x = 0;
@@ -3016,11 +3081,13 @@ gdk_event_translate (GdkDisplay *display,
 	  if (msg->wParam == SIZE_RESTORED)
 	    gdk_synthesize_window_state (window,
 					 GDK_WINDOW_STATE_ICONIFIED |
-					 GDK_WINDOW_STATE_MAXIMIZED,
+					 GDK_WINDOW_STATE_MAXIMIZED |
+					 withdrawn_bit,
 					 0);
 	  else if (msg->wParam == SIZE_MAXIMIZED)
 	    gdk_synthesize_window_state (window,
-					 GDK_WINDOW_STATE_ICONIFIED,
+					 GDK_WINDOW_STATE_ICONIFIED |
+					 withdrawn_bit,
 					 GDK_WINDOW_STATE_MAXIMIZED);
 
 	  if (private->resize_count > 1)
@@ -3098,7 +3165,7 @@ gdk_event_translate (GdkDisplay *display,
     case WM_MOVE:
       GDK_NOTE (EVENTS, g_print ("WM_MOVE: %p  (%d,%d)\n",
 				 msg->hwnd,
-				 LOWORD (msg->lParam), HIWORD (msg->lParam)));
+				 GET_X_LPARAM (msg->lParam), GET_Y_LPARAM (msg->lParam)));
 
       if (!(private->event_mask & GDK_STRUCTURE_MASK))
 	break;
@@ -3109,8 +3176,8 @@ gdk_event_translate (GdkDisplay *display,
 	{
 	  event->configure.type = GDK_CONFIGURE;
 	  event->configure.window = window;
-	  event->configure.x = LOWORD (msg->lParam);
-	  event->configure.y = HIWORD (msg->lParam);
+	  event->configure.x = GET_X_LPARAM (msg->lParam);
+	  event->configure.y = GET_Y_LPARAM (msg->lParam);
 	  GetClientRect (msg->hwnd, &rect);
 	  event->configure.width = rect.right;
 	  event->configure.height = rect.bottom;
