@@ -139,7 +139,7 @@ gtk_cell_renderer_toggle_class_init (GtkCellRendererToggleClass *class)
 		                   PROP_INCONSISTENT,
 				   g_param_spec_boolean ("inconsistent",
 					                 _("Inconsistent state"),
-							 _("The inconsistent stae of the button"),
+							 _("The inconsistent state of the button"),
 							 FALSE,
 							 G_PARAM_READABLE |
 							 G_PARAM_WRITABLE));
@@ -463,5 +463,5 @@ gtk_cell_renderer_toggle_set_active (GtkCellRendererToggle *toggle,
 {
   g_return_if_fail (GTK_IS_CELL_RENDERER_TOGGLE (toggle));
 
-  g_object_set (G_OBJECT (toggle), "active", setting?TRUE:FALSE, NULL);
+  g_object_set (toggle, "active", setting ? TRUE : FALSE, NULL);
 }
