@@ -22,10 +22,10 @@
 #include <string.h>
 #include <locale.h>
 
-#include "gtksignal.h"
 #include "gtkimmulticontext.h"
 #include "gtkimmodule.h"
 #include "gtkradiomenuitem.h"
+#include "gtkintl.h"
 
 static void     gtk_im_multicontext_class_init         (GtkIMMulticontextClass  *class);
 static void     gtk_im_multicontext_init               (GtkIMMulticontext       *im_multicontext);
@@ -77,10 +77,10 @@ static GtkIMContextClass *parent_class;
 
 static const gchar *global_context_id = NULL;
 
-GtkType
+GType
 gtk_im_multicontext_get_type (void)
 {
-  static GtkType im_multicontext_type = 0;
+  static GType im_multicontext_type = 0;
  
   if (!im_multicontext_type)
     {
@@ -97,9 +97,9 @@ gtk_im_multicontext_get_type (void)
         (GInstanceInitFunc) gtk_im_multicontext_init,
       };
       
-      im_multicontext_type = g_type_register_static (GTK_TYPE_IM_CONTEXT,
-						     "GtkIMMulticontext",
-						     &im_multicontext_info, 0);
+      im_multicontext_type =
+	g_type_register_static (GTK_TYPE_IM_CONTEXT, "GtkIMMulticontext",
+				&im_multicontext_info, 0);
     }
 
   return im_multicontext_type;
@@ -143,7 +143,7 @@ gtk_im_multicontext_init (GtkIMMulticontext *multicontext)
 GtkIMContext *
 gtk_im_multicontext_new (void)
 {
-  return GTK_IM_CONTEXT (g_object_new (GTK_TYPE_IM_MULTICONTEXT, NULL));
+  return g_object_new (GTK_TYPE_IM_MULTICONTEXT, NULL);
 }
 
 static void
@@ -167,16 +167,16 @@ gtk_im_multicontext_set_slave (GtkIMMulticontext *multicontext,
 	gtk_im_context_reset (multicontext->slave);
       
       g_signal_handlers_disconnect_by_func (multicontext->slave,
-					    (gpointer) gtk_im_multicontext_preedit_start_cb,
+					    gtk_im_multicontext_preedit_start_cb,
 					    multicontext);
       g_signal_handlers_disconnect_by_func (multicontext->slave,
-					    (gpointer) gtk_im_multicontext_preedit_end_cb,
+					    gtk_im_multicontext_preedit_end_cb,
 					    multicontext);
       g_signal_handlers_disconnect_by_func (multicontext->slave,
-					    (gpointer) gtk_im_multicontext_preedit_changed_cb,
+					    gtk_im_multicontext_preedit_changed_cb,
 					    multicontext);
       g_signal_handlers_disconnect_by_func (multicontext->slave,
-					    (gpointer) gtk_im_multicontext_commit_cb,
+					    gtk_im_multicontext_commit_cb,
 					    multicontext);
 
       g_object_unref (multicontext->slave);
@@ -450,7 +450,7 @@ activate_cb (GtkWidget         *menuitem,
 {
   if (GTK_CHECK_MENU_ITEM (menuitem)->active)
     {
-      const gchar *id = gtk_object_get_data (GTK_OBJECT (menuitem), "gtk-context-id");
+      const gchar *id = g_object_get_data (G_OBJECT (menuitem), "gtk-context-id");
 
       gtk_im_context_reset (GTK_IM_CONTEXT (context));
       
@@ -473,7 +473,7 @@ gtk_im_multicontext_append_menuitems (GtkIMMulticontext *context,
 				      GtkMenuShell      *menushell)
 {
   const GtkIMContextInfo **contexts;
-  gint n_contexts, i;
+  guint n_contexts, i;
   GSList *group = NULL;
   
   _gtk_im_module_list (&contexts, &n_contexts);
@@ -481,9 +481,36 @@ gtk_im_multicontext_append_menuitems (GtkIMMulticontext *context,
   for (i=0; i < n_contexts; i++)
     {
       GtkWidget *menuitem;
-
+      const gchar *translated_name;
+#ifdef ENABLE_NLS
+      if (contexts[i]->domain && contexts[i]->domain_dirname &&
+	  contexts[i]->domain[0] && contexts[i]->domain_dirname[0])
+	{
+	  if (strcmp (contexts[i]->domain, GETTEXT_PACKAGE) == 0 &&
+	      strcmp (contexts[i]->domain_dirname, GTK_LOCALEDIR) == 0)
+	    /* Input method may have a name in the GTK+ message catalog */
+	    translated_name = _(contexts[i]->context_name);
+	  else
+	    /* Input method has own message catalog */
+	    {
+	      bindtextdomain (contexts[i]->domain,
+			      contexts[i]->domain_dirname);
+#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
+	      bind_textdomain_codeset (contexts[i]->domain, "UTF-8");
+#endif
+	      translated_name = dgettext (contexts[i]->domain, contexts[i]->context_name);
+	    }
+	}
+      else
+	/* Either domain or domain_dirname is NULL or "". We assume that
+	 * input method does not want a translated name in this case
+	 */
+	translated_name = contexts[i]->context_name;
+#else
+      translated_name = contexts[i]->context_name;
+#endif
       menuitem = gtk_radio_menu_item_new_with_label (group,
-                                                     contexts[i]->context_name);
+						     translated_name);
       
       if ((global_context_id == NULL && group == NULL) ||
           (global_context_id &&
@@ -491,12 +518,12 @@ gtk_im_multicontext_append_menuitems (GtkIMMulticontext *context,
         gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem),
                                         TRUE);
       
-      group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
+      group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menuitem));
       
-      gtk_object_set_data (GTK_OBJECT (menuitem), "gtk-context-id",
-			   (char *)contexts[i]->context_id);
-      gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
-			  G_CALLBACK (activate_cb), context);
+      g_object_set_data (G_OBJECT (menuitem), "gtk-context-id",
+			 (char *)contexts[i]->context_id);
+      g_signal_connect (menuitem, "activate",
+			G_CALLBACK (activate_cb), context);
 
       gtk_widget_show (menuitem);
       gtk_menu_shell_append (menushell, menuitem);
