@@ -44,7 +44,7 @@
 static char *
 escape_string (const char *str)
 {
-  GString *result = g_string_new ("");
+  GString *result = g_string_new (NULL);
 
   while (TRUE)
     {
